
/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 1999, 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by Vovida Networks,
 *        Inc. (http://www.vovida.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must 
 *    not be used to endorse or promote products derived from this 
 *    software without prior written permission. For written permission, 
 *    please contact vocal@vovida.org.
 *
 * 5. Products derived from this software may not be called "VOCAL",
 *    nor may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL VOVIDA NETWORKS, INC. OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida 
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see 
 * <http://www.vovida.org/>.
 *
 */





static const char* const AgentApi_cxx_Version = 
"$Id: AgentApi.cxx,v 1.20 2000/08/10 11:16:53 bogawa Exp $";
#include <unistd.h>
#include <sys/param.h>
#include <netinet/in.h>
#include "AgentApi.hxx"
#include "UdpStack.hxx"
#include "cpLog.h"

AgentApi::AgentApi(ServerType serType = Unknown, string appName = "unknown" ) : ThreadIf()
{
    char hostname[MAXHOSTNAMELEN];

    myServerType = serType;
    myApplName = appName;

    memset(hostname, 0, sizeof(hostname));
    if (gethostname(hostname, MAXHOSTNAMELEN) < 0) {
	agentIpStr.assign("");
    } else {
	agentIpStr.assign(hostname);
    }

}


///
AgentApi::~AgentApi()
{
// close ports, deallocate memory
	cpLog( LOG_DEBUG, "entered agentApi destructor");
	shutdown();
	join();
}

///
voReturnStatus
AgentApi::sendTrap(int trapType, string parameter)
{
    try {
	NetworkAddress dest(agentIpStr, agentTrapPort);
	memset(&trapMessage, 0, sizeof(trapMessage));
	trapMessage.action = (actionT)Trap;
	trapMessage.mibVariable = (AgentApiMibVarT)trapType;
	if (parameter.length() < PARM1SIZE ) {
	    strcpy(trapMessage.parm1, parameter.c_str());
	} else {
	    return voFailure;
	}
	udpStack->transmitTo((char *)&trapMessage, sizeof(trapMessage), &dest);
	return voSuccess;
    }
    catch ( ... ) {
	return voFailure;
    }
}

voReturnStatus
AgentApi::sendResponse(int val, NetworkAddress *sender)
{
    try {
	int *parm2 = (int *)message.parm2;
	message.action = (actionT)Response;
	*parm2 = val;
	cpLog( LOG_DEBUG, "send integer response: %d to %s:%d", 
	       val, sender->getHostName().c_str(), sender->getPort());
	udpStack->transmitTo((char *)&message, sizeof(message), sender);
	return voSuccess;
    }
    catch ( ... ) {
	return voFailure;
    }
    return voSuccess;
}

voReturnStatus
AgentApi::sendResponse(ulong val, NetworkAddress *sender)
{
    try {
	int *parm2 = (int *)message.parm2;
	message.action = (actionT)Response;
	*parm2 = val;
	cpLog( LOG_DEBUG, "send ulong response: %d to %s:%d", 
	       val, sender->getHostName().c_str(), sender->getPort());
	udpStack->transmitTo((char *)&message, sizeof(message), sender);
	return voSuccess;
    }
    catch ( ... ) {
	return voFailure;
    }
    return voSuccess;
}

voReturnStatus
AgentApi::sendResponse(string parameter, NetworkAddress *sender)
{
    try {
	message.action = (actionT)Response;
	if (parameter.length() < PARM2SIZE ) {
	    strcpy(message.parm2, parameter.c_str());
	}
	else {
	    return voFailure;
	}
	cpLog( LOG_DEBUG, "send string response: %s to %s:%d", 
	       parameter.c_str(), sender->getHostName().c_str(), sender->getPort());
	udpStack->transmitTo((char *)&message, sizeof(message), sender);
	return voSuccess;
    }
    catch ( ... ) {
	return voFailure;
    }
}

voReturnStatus
AgentApi::sendResponse(void *inData, NetworkAddress *sender)
{
    try {
	message.action = (actionT)Response;
	memcpy(message.parm2, inData, PARM2SIZE);
	cpLog( LOG_DEBUG, "send generic response to %s:%d", 
	       sender->getHostName().c_str(), sender->getPort());
	udpStack->transmitTo((char *)&message, sizeof(message), sender);
	return voSuccess;
    }
    catch ( ... ) {
	return voFailure;
    }
}

voReturnStatus 
sendRequest(string indexName, ServerType serType)
{
    cpLog( LOG_ERR, "Using function not yet implemented 2" );
    return voFailure;
}

voReturnStatus 
sendRequest(string indexName, int setValue)
{
    cpLog( LOG_ERR, "Using function not yet implemented 2" );
    return voFailure;
}

void
AgentApi::thread() 
{
    int bytesRead = 0; 
    NetworkAddress sender;
    NetworkAddress dest(registerMulticastIP, registerMulticastPort);

    myLock.lock();

    switch (myServerType) {
	case Agent:
	    // register to receive the messages on the well known agent port
	    // then send out the multicast message.
	    udpStack = new UdpStack(NULL, agentTrapPort, agentTrapPort, sendrecv, false);
	    message1.action = (actionT)Register_Req;
	    udpStack->transmitTo((char *)&message1, sizeof(message1), &dest);
	    break;

	case NetMgnt:
	    udpStack = new UdpStack(NULL,MANAGERTRAPPORT , MANAGERTRAPPORT, sendrecv, false);
	    break;

	default :

	    udpStack = new UdpStack(NULL,-1 ,-1 , sendrecv, false);
	
	    message1.action = (actionT)Register;
	    int *pi;
	    pi = (int *)message1.parm1;
	    pi[0] = (int)myServerType;
	    pi[1] = udpStack->getRxPort();
	    memcpy(message1.parm2,(char*)myApplName.c_str(), PARM2SIZE); 
	
	    agentRegister = new AgentRegister((void *)&message1,sizeof(message1));
	    agentRegister->run();
	    break;
    }
	
    myLock.unlock();

    while (true)
    {
        try
        {
	    // Poll every fifteen seconds
	    //
            memset((void *)&message, 0, sizeof(ipcMessage));
            bytesRead = udpStack->receiveTimeout((char *)&message, sizeof(ipcMessage), &sender, 15);
	    if ( bytesRead )
	    {
                cpLog( LOG_DEBUG, "bytes read=%d",bytesRead);  
	        processMessage(&message, &sender);
	    }
        }
        catch ( ... )
        {
            cpLog( LOG_ERR, "Exception from udpstack for AgentApiRx" );
        }
	
	if ( isShutdown() == true )
	{
	    return;
	}
    }
}
