
/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 1999, 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by Vovida Networks,
 *        Inc. (http://www.vovida.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must 
 *    not be used to endorse or promote products derived from this 
 *    software without prior written permission. For written permission, 
 *    please contact vocal@vovida.org.
 *
 * 5. Products derived from this software may not be called "VOCAL",
 *    nor may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL VOVIDA NETWORKS, INC. OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida 
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see 
 * <http://www.vovida.org/>.
 *
 */





static const char* const AgentRegister_cxx_Version = 
"$Id: AgentRegister.cxx,v 1.8 2000/08/10 11:16:53 bogawa Exp $";
#include <unistd.h>
#include <sys/param.h>
#include <netinet/in.h>
#include "AgentRegister.hxx"
#include "UdpStack.hxx"
#include "cpLog.h"

AgentRegister::AgentRegister(void *msg,int msgLEN) : ThreadIf()
{
    const int maxHostNameLen = 128;
	char hostname[maxHostNameLen];
	txMsgLen = msgLEN;
	txMessage = (char *)calloc(txMsgLen, sizeof(char));
	memcpy(txMessage, msg, txMsgLen);
	memset(hostname, 0, sizeof(hostname));
	if (gethostname(hostname,maxHostNameLen ) < 0) {
	    cpLog(LOG_ERR, "gethostname failed: %s", strerror);
	    throw "big fat mess";
	}
	NetworkAddress mcAddr(registerMulticastIP,registerMulticastPort );
	dest.setHostName(hostname);
	dest.setPort(agentTrapPort);

	regUdpStack = new UdpStack(&mcAddr, registerMulticastPort, registerMulticastPort, sendrecv, false, true);
	if (regUdpStack == 0) {
	    cpLog(LOG_ERR, "can't register udp multicast port %d", registerMulticastPort);
	    throw"another bad thing happened";
	}
	NetworkAddress iface(hostname);
	regUdpStack->joinMulticastGroup(mcAddr, &iface, 0);

	regUdpStack->transmitTo(txMessage,txMsgLen, &dest);	
}                       

AgentRegister::~AgentRegister()
{
    if (txMessage) free (txMessage);
}

void
AgentRegister::thread()
{
    int bytesRead = 0;
    NetworkAddress sender;

    while (true)
    {
        try
        {
            // Poll every 5 minutes

            memset((void *)&rxMessage, 0, sizeof(rxMessage));
            bytesRead = regUdpStack->receiveTimeout((char *)&rxMessage,sizeof(rxMessage), &sender, 300);

		// There are 3 cases that need to be take care of.
		// 1. Timeout - send the message to the trapport (0)
		// 2. received response - send message to the trapport (>0)
		// 3. an error occured ..... just report an error (-1)
		// If the message we get is either a 0 or greater than that then send a message with 
		// the port number and type, name of app, to the snmpd at the port 33602 (trapport ) on which the 
		// snmpd will be listening ....
		// Also I will have to construct the message in the constructor.

            if ( bytesRead >= 0)
            {
		regUdpStack->transmitTo(txMessage,txMsgLen, &dest);
            }
            if ( bytesRead == -1)
            {
                cpLog( LOG_ERR, "Error in receiving on port %s", regUdpStack->getRxPort());
            }
        }
        catch ( ... )
        {
            cpLog( LOG_ERR, "Exception from udpstack for AgentRegister" );
        }
        if ( isShutdown() == true )
        {
		return;
        }
    }
}               
