
/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 1999, 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by Vovida Networks,
 *        Inc. (http://www.vovida.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must 
 *    not be used to endorse or promote products derived from this 
 *    software without prior written permission. For written permission, 
 *    please contact vocal@vovida.org.
 *
 * 5. Products derived from this software may not be called "VOCAL",
 *    nor may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL VOVIDA NETWORKS, INC. OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida 
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see 
 * <http://www.vovida.org/>.
 *
 */





static const char* const CryptoRandom_cxx_Version = 
"$Id: CryptoRandom.cxx,v 1.5 2000/08/10 11:16:54 bogawa Exp $";

    
#if USE_CRYPTO_RANDOM
    
    
#include "CryptoRandom.hxx"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>    
    
#include <errno.h> 
    

#include <unistd.h>     
#include "cpLog.h"


CryptoRandom::CryptoRandom()
{
    //open the io device /dev/random.
    fd = open( "/dev/random", O_RDONLY );
    
    
    
    
    
    if ( fd == -1 )
    {
	cpLog(LOG_DEBUG, "CryptoRandom::unable to open /dev/random");
        switch(errno)
	{
	    case EACCES: 
	    {
		cpLog(LOG_DEBUG, "Error is: no permissions");
		break;
	    }
	    case ENXIO: 
	    {
		cpLog(LOG_DEBUG, "device does not exist");
		break;
	    }
             case EAGAIN: 
	    {
		cpLog(LOG_DEBUG, "Error is: resource not available");
		break;
	    }
	    case EEXIST:
	    {
		cpLog(LOG_DEBUG, "EEXIST");
		break;
	    }
             case EISDIR:
	    {
		cpLog(LOG_DEBUG, "EISDIR");
		break;
	    }
             case ENAMETOOLONG:
	    {
		cpLog(LOG_DEBUG, "ENAMETOOLONG");
		break;
	    }
             case ENOENT:
	    {
		cpLog(LOG_DEBUG, "ENOENT");
		break;
	    }
	     case ENOTDIR:
	    {
		cpLog(LOG_DEBUG, "ENOTDIR");
		break;
	    }
            case EROFS:
	    {
		cpLog(LOG_DEBUG, "EROFS");
		break;
	    }
    
            default:
	        cpLog(LOG_DEBUG,  "error num in opening is: %d", errno);
		break;
            
	}
    
      
    }
 
}
    
CryptoRandom::~CryptoRandom()
{
    
    //close the io device /dev/random.
    int err = close(fd);
    if (err == -1)
    {
	cpLog(LOG_DEBUG, "Error occured when closing the /dev/random fd");
    }
}
    
//char* CryptoRandom::getRandom()
int CryptoRandom::getRandom(unsigned char* buf, int num)
{
    //read from /dev/random, and return.
    
    
    /*
    octl(fd, RNDADDTOENTCNT, &amount); to increment the
    amount of entropy, or
    struct entropy 
    {
    int bits;          Bits of entropy contained in buf 
    int size;
    char buf[size]
    }
    */
    
    

    
#if 0
    int err = ioctl(fd, RNDGETENTCNT, &entropy);
    
//    int err = ioctl(fd, RNDGETPOOL, &entropy);
    //got EINVAL for this.
#endif
   
     
     ssize_t len = read(fd, buf, num); 
    
     buf[num] = '\0';
    
    if (len == -1)
    {
	cpLog(LOG_DEBUG,  "CryptoRandom:: error no .  in reading: %d" ,errno );
#if 0 
    
           switch(errno)
	{
	    case EBADF: 
	    {
		cpLog(LOG_DEBUG, "EBADF");
		break;
	    }
	    case EFAULT: 
	    {
		cpLog(LOG_DEBUG, "EFAULT");
		break;
	    }
             case ENOTTY: 
	    {
		cpLog(LOG_DEBUG, "ENOTTY");
		break;
	    }
	    case EINVAL:
	    {
		cpLog(LOG_DEBUG, "EINVAL");
		break;
	    }
	    case EACCES:
	    {
		cpLog(LOG_DEBUG, "EACCES");
		break;
	    }
          
	    default:
		break;
	}
#endif
    }
    else
    {
    
        cpLog(LOG_DEBUG, " CryptoRandom::got random numbers");
    }
    
    return len;
    
    
    
}
    

    

#endif
