static const char* const NetworkAddress_cxx_fileVersion = 
 "$Id: NetworkAddress.cxx,v 1.28 2000/06/19 20:04:52 marcl Exp $";

/*
* Copyright (c) 1999-2000 Mera Ltd.
* Copyright 1999-2000 Vovida Networks, Inc.
*
* The contents of this file are subject to the Mozilla Public License
* Version 1.0 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS"
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
* the License for the specific language governing rights and limitations
* under the License.
*
* Portions of this code were written with the assisance of funding from
* Vovida Networks, Inc. http://www.vovida.com.
*
* Portions are Copyright (C) 1993 Free Software Foundation, Inc.
* All Rights Reserved.
*
*/

// $Log: NetworkAddress.cxx,v $
// Revision 1.28  2000/06/19 20:04:52  marcl
// Make getIpName() thread-safe - Surendra
//
// Revision 1.27  2000/06/08 23:25:44  surendra
// Print log message when can not get hostName
//
// Revision 1.26  2000/06/07 20:40:49  bogawa
// fixes to make hostmatch threadsafe with networkaddress
//
// Revision 1.25  2000/06/07 19:18:25  surendra
//
// Mutexed the gethostbyname function.
//
// Revision 1.24  2000/05/17 19:46:23  wenqing
// fix for gethostname
//
// Revision 1.23  2000/05/15 23:50:27  wenqing
// add one function for get sockaddr type of address
//
// Revision 1.22  2000/05/01 19:06:14  bogawa
// fixes for FreeBSD compiling in util directory
//
// Revision 1.21  2000/04/17 21:56:25  rag
// Added hashIpPort()
//
// Revision 1.20  2000/03/23 23:24:09  wenqing
// Solving mem leak of strstream usage
//
// Revision 1.19  2000/03/11 04:31:22  jeff
// fix for  operator==  --- Wenqing Jin
//
// Revision 1.18  2000/02/24 18:32:22  bogawa
// fixes for VxWorks Compile
//
// Revision 1.17  2000/02/24 02:18:51  hong
// mplemented at-most-once functionality/re-send on timeout in MGCP
//
// Revision 1.16  2000/02/18 00:09:45  wenqing
// take out some include file
//
// Revision 1.15  2000/02/02 01:54:25  hong
// Replacing MGCP stack's own UDP implementation with the UDP stack in util
//
// Revision 1.14  2000/01/31 20:21:56  wenqing
// update for stack mode and some misc items
//
// Revision 1.13  2000/01/11 00:56:00  bogawa
// copyright notice fixes for year 2000
//
// Revision 1.12  1999/12/24 02:58:01  tina
// remove #ifdef __vxworks, make it to OS independent.
//
// Revision 1.11  1999/12/21 21:51:46  bogawa
// fixes for the great renaming
//
// Revision 1.10  1999/12/18 04:28:48  eckelcu
// Modified Files:
// 	networkAddress.cxx udp.cxx test/VThreadTest.cxx
// 	test/udpTest/Makefile test/udpTest/test_udp.cxx
// Changes for udp to work for vxworks
//
// Revision 1.9  1999/12/16 20:32:20  wenqing
// fix up some bugs while write test programs
//
// Revision 1.8  1999/11/19 00:38:00  bogawa
// changes for the util directory as well as using the new vmutex, etc. classes
//
// Revision 1.7  1999/11/17 22:52:37  wenqing
// use cpLog instead of cerr
//
// Revision 1.6  1999/11/12 21:47:09  wenqing
// more changes according to cullen's change in networkAddress class
//
// Revision 1.5  1999/11/12 16:26:23  cullen
// few changes from cullen
//
// Revision 1.4  1999/11/12 03:09:56  wenqing
// More complete update version. They are compiled.
//
// Revision 1.3  1999/11/11 18:36:53  wenqing
// File format change.
//
// Revision 1.2  1999/11/11 18:34:01  wenqing
// This is the first version. Ask for comments.
//

/*
#include <unistd.h>
#include <string.h>
*/

#include <string>
#if defined(__FreeBSD__)
#include <sys/types.h>
#endif
#include <netinet/in.h>
#include <arpa/inet.h>
#include <strstream.h>
#include <netdb.h>
#include "NetworkAddress.h"
#include "cpLog.h"
#include "vsock.hxx"
#include "LockHelper.hxx"
#include <iostream>

VMutex NetworkAddress::_mutex;

//------------------------------------------------------------------------------
// NetworkAddress construstor
//
NetworkAddress::NetworkAddress ( int port )
{
    /* The host is undefined yet.*/
    setHostName("localhost");  // assuming it is localhost first
    aPort = port;
}


NetworkAddress::NetworkAddress ( string hostName, int port = -1 )
{
    setPort(port);
    setHostName(hostName);
}


//------------------------------------------------------------------------------
//NetworkAddress set value method
//
void 
NetworkAddress::setHostName ( const string& theAddress )
{
    const char* cstrAddress;
    struct hostent * pxHostEnt = 0;
    string sAddress, sPort;
    unsigned int length = theAddress.length();
    unsigned int colonPos = length; 

    cstrAddress = theAddress.c_str();



    for (int i=0; *cstrAddress != '\0'; cstrAddress++, i++)
    {
        if (*cstrAddress == ':')
        {
            colonPos = i;
            break;
        }
    }

    if (colonPos != length)
    {
//        cpLog (LOG_INFO, "There is a colon\n");
        sAddress=theAddress.substr(0,colonPos);
        sPort=theAddress.substr(colonPos+1);

        cpLog (LOG_INFO, "Address[%s]\n", sAddress.c_str());
        cpLog (LOG_INFO, "Port[%s]\n", sPort.c_str());

        colonPos = atoi(sPort.c_str());
        if(colonPos) 
        {
            aPort = colonPos; 
            cpLog (LOG_INFO, "aPort = %d\n", aPort);
        }
        else
        {
            cpLog (LOG_ERR, "Incorrect Port" );

            cerr << "Incorrect Port" << endl;

        }
    }
    else // No ':' in input string;
    {
//        cpLog (LOG_INFO, "There is not a colon\n");
        sAddress = theAddress;
    }

    LockHelper helper(_mutex);
    pxHostEnt = gethostbyname(sAddress.c_str());

    if (pxHostEnt)
    {
        memcpy((void *)&ipAddress, 
	       (void *)pxHostEnt->h_addr, 
	       pxHostEnt->h_length);
    }
    else 
    {
//        cpLog (LOG_ERR, "Incorrect Address" );


    }
}


void 
NetworkAddress::setPort( int iPort )
{
    aPort = iPort;
}


//------------------------------------------------------------------------------
//NetworkAddress get value method
//
string 
NetworkAddress::getHostName( ) const
{
    string hostName;
    struct hostent * pxHostEnt = 0;

    _mutex.lock();

    pxHostEnt = gethostbyaddr(ipAddress,IPV4_LENGTH,AF_INET);

    if (pxHostEnt)
    {
      hostName = pxHostEnt->h_name;
      _mutex.unlock();
    }
    else
    {
      _mutex.unlock();
      hostName = getIpName();
    }

    return hostName;
}


string 
NetworkAddress::getIpName () const
{
    string ipName;
    struct in_addr temp;

    LockHelper helper(_mutex);

    memcpy((void *)&temp.s_addr,ipAddress,IPV4_LENGTH);
    ipName = inet_ntoa(temp);

    return ipName;
}


// This method is only good for ipv4
// It returns Ip address in network byte order
u_int32_t
NetworkAddress::getIp4Address () const
{
    u_int32_t lTmp;

    memcpy((void *) &lTmp,ipAddress,IPV4_LENGTH);
    return lTmp;
}

void
NetworkAddress::getSockAddr (struct sockaddr & socka) const
{
    short tmp_s = htons (aPort); 
    char * tmp_p;

    tmp_p = (char*) &tmp_s;
    socka.sa_family = AF_INET;
    socka.sa_data[0] = tmp_p[0];
    socka.sa_data[1] = tmp_p[1];
    memcpy((void *)&socka.sa_data[2], ipAddress, IPV4_LENGTH);

    return;
}


int  
NetworkAddress::getPort () const
{
    return aPort;
}


//NetworkAddress address to char translation
//
/*
void NetworkAddress::addressToChar ( char * pcAddress ) const
{
    memcpy(pcAddress,m_pcAddress,4);
}
*/

//NetworkAddress operator []
//
/*
unsigned char NetworkAddress::operator [] ( int nth ) const
{
    return m_pcAddress[nth];
}
*/

//NetworkAddress::operator < - Compare two NetworkAddresses
//
bool 
operator < ( const NetworkAddress & xAddress,
             const NetworkAddress & yAddress )
{
    for (int i = 0 ; i<= IPV4_LENGTH; i++ )
    {
        if ( xAddress.ipAddress[i] < yAddress.ipAddress[i]) 
        {
            return true;
        }
        if ( xAddress.ipAddress[i] > yAddress.ipAddress[i]) 
        {
            return false;
        }
    }
    return xAddress.aPort < yAddress.aPort ;  
}

//NetworkAddress::operator == - Compare two NetworkAddresses
//
bool 
operator == ( const NetworkAddress & xAddress,
              const NetworkAddress & yAddress )
{
    for (int i = 0 ; i < IPV4_LENGTH ; i++ )
    {
        if ( xAddress.ipAddress[i] != yAddress.ipAddress[i]) 
        {
            return false;
        }
    }
    return xAddress.aPort == yAddress.aPort ;  
}

/// assignment operator =
NetworkAddress& NetworkAddress::operator=( const NetworkAddress& x )
{
    this->aPort = x.aPort;
    memcpy( this->ipAddress, x.ipAddress, sizeof(this->ipAddress) );

    //   strncpy( this->ipAddress, x.ipAddress, IPV4_LENGTH );
    return ( *this );
}


//NetworkAddress typecast to std::string
//
NetworkAddress::operator string () 
{
    string xStr;
    strstream xStream;
    xStream << (*this);
    xStr=xStream.str();
    xStream.freeze(false);
    return xStr;
}


ostream & 
NetworkAddress::print ( ostream & xStr ) const
{
    struct hostent * pxHostEnt;
    struct in_addr temp;
    memcpy((void *)&temp.s_addr,ipAddress,IPV4_LENGTH);
    char * cpBuf = (char *)inet_ntoa(temp);

    LockHelper helper(_mutex);

    pxHostEnt = gethostbyaddr(ipAddress,IPV4_LENGTH,AF_INET);

    if ( pxHostEnt )
    {
        xStr << pxHostEnt->h_name << "(" << cpBuf << ")";
    }
    else
    {
        xStr << cpBuf ; 
    }
    if ( aPort ) 
        xStr << ":" << aPort;

    return xStr;
}

///
u_int32_t
NetworkAddress::hashIpPort( )
{
    u_int32_t ipAddress = getIp4Address ();
    int port = getPort ();

    return (hashIpPort( ipAddress, port )) ;
}
   
u_int32_t
NetworkAddress::hashIpPort( const string& theAddress, const string& port ) 
{
    static NetworkAddress net;

    net.setHostName ( theAddress );
    net. setPort ( atoi(port.c_str()) );

    return ( net.hashIpPort() );
}

u_int32_t
NetworkAddress::hashIpPort( const u_int32_t ipAddress, const int port )
{
    u_int32_t hashKey = 0x0;
    hashKey = (ipAddress<<16) | (port & 0xFF);

    return hashKey;
}

int 
NetworkAddress::getHostByName(const char* hostName, 
			      struct hostent* hEnt,
			      char* buf,
			      int buflen,
			      int* thrErrno)
{
#if defined(__linux__)
    struct hostent* x;
    int retval = gethostbyname_r(hostName, hEnt, buf, buflen, &x, thrErrno);
    if (retval == 0)
	return getHostLookupOK;
 
    cpLog(LOG_DEBUG, "********* Failed to get host by name:%s", hostName); 
        
    return getHostLookupFailed;

#else
 #if defined(__svr4__)
    struct hostent* x;
    x = gethostbyname_r(hostName, hEnt, buf, buflen, thrErrno);
    if(x == 0)
    {
        cpLog(LOG_DEBUG, "********* Failed to get host by name:%s", hostName); 
        return getHostLookupFailed;
    }
    return getHostLookupOK;
 #else
  #error no implementation for critical function
 #endif
#endif

}


// End of File
