#ifndef SIGNAL_SET_DOT_H
#define SIGNAL_SET_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 1999, 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by Vovida Networks,
 *        Inc. (http://www.vovida.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must 
 *    not be used to endorse or promote products derived from this 
 *    software without prior written permission. For written permission, 
 *    please contact vocal@vovida.org.
 *
 * 5. Products derived from this software may not be called "VOCAL",
 *    nor may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL VOVIDA NETWORKS, INC. OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida 
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see 
 * <http://www.vovida.org/>.
 *
 */





static const char* const SignalSetHeaderVersion = 
    "$Id: SignalSet.hxx,v 1.4 2000/08/10 11:16:55 bogawa Exp $";




#include <signal.h>
#include <vector>
#include "Writer.hxx"

namespace Vovida
{

class SignalSet : public Writer
{
    public:
    
    	SignalSet();
	SignalSet(int signum);
	SignalSet(int *, size_t);
	SignalSet(const vector<int> &);

	SignalSet(const SignalSet &);
				    
	~SignalSet();

	SignalSet & operator=(const SignalSet &);

    	void	    	    	    fill();
	void	    	    	    empty();
	void	    	    	    insert(int);
	void	    	    	    insert(int *, size_t);
	void	    	    	    insert(const vector<int> &);
	void	    	    	    erase(int);
	void	    	    	    erase(int *, size_t);
	void	    	    	    erase(const vector<int> &);
    	bool	    	    	    contains(int) const;
	bool	    	    	    contains(int *, size_t) const;
	bool	    	    	    contains(const vector<int> &) const;
	
    	const vector<int> & 	    signalNumbers() const;
	
    	sigset_t    	    	    signals;

	virtual ostream &           writeTo(ostream &) const;

    private:
    
    	vector<int> 	    	    signalNumbers_;
    
    	// Suppress copying, since the os does provide copy functionality.
	//
};

} // namespace Vovida

 // !defined(SIGNAL_SET_DOT_H)
#endif
